"""Files and directories access.

The :class:`File` and :class:`Directory` classes provide abstractions to access
filesytem entities.

"""

import os
from pathlib import PosixPath


class Path:
    """A filesystem path such as a file or directory."""

    def __init__(self, path):
        self._path = PosixPath(path).absolute()

    @property
    def name(self):
        return self._path.name

    @property
    def exists(self):
        """Whether the path exists."""
        return self._path.exists()

    @property
    def readable(self):
        """Whether the path is readable."""
        return os.access(str(self._path), os.R_OK)

    @property
    def writable(self):
        """Whether the path is writable."""
        return os.access(str(self._path), os.W_OK)


class File(Path):
    """Wrapper to reaad/write a file."""

    def read(self):
        """Return file content."""
        return self._path.read_text()

    def write(self, content):
        """Write content to file, replacing the content if it exists."""
        self._path.write_text(content)


class Directory(Path):
    """Access  files in a directory with a :class:`dict`-like interface."""

    #: Map names of files under the directory to their corresponding
    #: :class:`File` type.  Subclasses should define this.
    files = {}

    def list(self):
        """Return a list of names in the directory.

        Only existing files that match names listed in `files` are returned.

        """
        return sorted(
            name for name in self.files if (self._path / name).exists())

    def listdir(self):
        """Return all existing names in a directory."""
        return [path.name for path in self._path.iterdir()]

    def join(self, *paths):
        """Append the given path to the directory one."""
        return self._path.joinpath(*paths)

    def __getitem__(self, name):
        """Return the :class:`File` instance for a name."""
        item = self.files[name](self._path / name)
        if not item.exists:
            raise KeyError(name)
        return item

    def __iter__(self):
        """Return an iterator yielding :class:`File`s in the directory."""
        for name in self.list():
            yield self[name]



from ..directory import ParsedDirectory
from ...testing import TestCase


class SampleParsedDirectory(ParsedDirectory):

    def _parse(self, path):
        return path.name + '-parsed'


class ParsedDirectoryTests(TestCase):

    def test_parse(self):
        """ParsedDirectory.parse calls the parser with path of each file."""
        self.tempdir.mkfile(path='foo')
        self.tempdir.mkfile(path='bar')
        parsed_dir = SampleParsedDirectory(self.tempdir.path)
        self.assertEqual(
            parsed_dir.parse(), {'foo': 'foo-parsed', 'bar': 'bar-parsed'})

    def test_parse_not_existent(self):
        """ParsedDirectory.parse returns None if directory doesn't exist."""
        parsed_dir = SampleParsedDirectory(self.tempdir.join('somedir'))
        self.assertIsNone(parsed_dir.parse())



"""Classes for directory content."""

from abc import (
    ABCMeta,
    abstractmethod,
)

from ..fs import Directory


class ParsedDirectory(Directory, metaclass=ABCMeta):
    """A directory whose file listing is parsed."""

    def parse(self):
        """Return a dict with files in the directory and their parse result."""
        if not self.exists:
            return

        return {path.name: self._parse(path) for path in self._path.iterdir()}

    @abstractmethod
    def _parse(self, path):
        """Parse details for a path in the directory.

        .. note::
            Subclasses must implement this method.

        """
